/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
 * http://www.gnu.org/software/gnugo/ for more information.          *
 *                                                                   *
 * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
 * by the Free Software Foundation.                                  *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 2             *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "gnugo.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dirent.h>
#include <math.h>

#include "liberty.h"
#include "interface.h"
#include "sgftree.h"
#include "gg_utils.h"

#define DEBUG_COMMANDS "\
capture <pos>    try to capture indicated group\n\
dead             Toggle display of dead stones\n\
defend <pos>     try to defend indicated group\n\
listdragons      print dragon info \n\
showarea         display area\n\
showdragons      display dragons\n\
showmoyo         display moyo\n\
showterri        display territory\n\
"

const char* szVerInfo = " ٵ v0.05 ȯ ";

#ifdef V43
const char *szSGFPath = "/mnt/hd/user/baduk/sgf/";
#else
const char *szSGFPath = "/home/v43/SDL_host/gnugo-3.6/sgf/";
#endif

#ifdef V43
const char *szIniPath = "/mnt/hd/user/baduk/data/";
#else
const char *szIniPath = "/home/v43/SDL_host/gnugo-3.6/data/";
#endif

#ifdef V43
//common
const char * szStoneWav = "/mnt/hd/user/baduk/data/stone.wav";
#else
//common
const char * szStoneWav = "./data/stone.wav";
#endif

#define STR_FILTER ".sgf"

const SDL_Color scWhite   = {0xFF,0xFF,0xFF,0x00};
const SDL_Color scBlack   = {0x00,0x00,0x00,0x00};
const SDL_Color scYellow	= {0xFF,0xF4,0x68,0x00};
const	SDL_Color scGray		=	{0xDC,0xDC,0xDC,0x00};//ȸ
const SDL_Color scGreen		= {0x17,0x7B,0x2F,0x00};

SDL_Rect srButton[NUM_BUTTON_CNT];
SDL_Rect srOption[2][NUM_OPTION_CNT];
SDL_Rect srThumb;
SDL_TimerID timer1;

FILELIST stFileList;
int iBtnCmd[NUM_BUTTON_CNT];
bool bPlayBlack=true;
bool bSound=true;
bool bSetKomi=true;
bool bForce=false;
bool bSetOption=true;
bool bRecordSee=false;//⺸ 
bool bTimer=false;
int		iSecond=1;
float fSetKomi=6.5;
int	 iBoardSize=19;
int  iHandiCap=0;
int  iLevel=10;
int	 iMoveNum=0;
int	 iMaxMoveNum=0;
//---------------------------------------------------------------------
/* some options for the ascii interface */
static int opt_showboard = 1;
static int showdead = 0;
static int emacs = 0;
static SGFTree sgftree;
//static int last_move_i;      /* The position of the last move */
//static int last_move_j;      /* -""-                          */
static int resignation_allowed;

/* Unreasonable score used to detect missing information. */
#define NO_SCORE 4711
/* Keep track of the score estimated before the last computer move. */
static int current_score_estimate = NO_SCORE;

static int ascii_endgame(Gameinfo *gameinfo, int reason);
static void ascii_count(Gameinfo *gameinfo);
static void showcapture(char *line);
static void showdefense(char *line);
static void ascii_goto(Gameinfo *gameinfo, char *line);
static void ascii_free_handicap(Gameinfo *gameinfo, char *handicap);

/* If sgf game info is written can't reset parameters like handicap, etc. */
static int sgf_initialized;

static int audio_opened=0;
static Mix_Chunk *mcStone=NULL;

int LoadAllWav()
{
	if ( Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 4096) < 0 ) {
		fprintf(stderr, " ġ   ϴ. : %s\n",
		SDL_GetError());
		return(-1);
	}
	/* ̺  ޸𸮿 ε */
	//common
	mcStone = Mix_LoadWAV(szStoneWav);
	if ( mcStone == NULL ) {
		fprintf(stderr, "̺  ε  %s: %s\n",
			szStoneWav, SDL_GetError());
		return 1;
	}
	
	return 0;
}

void FreeAllWav(void)
{
	/* wave   */
	//common
	if ( mcStone ) {
		Mix_FreeChunk(mcStone);
		mcStone = NULL;
	}
		
	/*  ġ  ִ ¶ ݱ */
	if ( audio_opened ) {
		Mix_CloseAudio();
		audio_opened = 0;
	}
}

int PlayWav( Mix_Chunk *pWave ,Uint8 nChannel)
{
	Mix_PlayChannel(nChannel, pWave, 0);
	/* ̶  Ŵ */
	while ( Mix_Playing(0) || Mix_Playing(1) ) {
		SDL_Delay(100);
	}

	return 0;
}

static void MakeFullFilePath(char *pPath, char *pName, char *pFullPath)
{
		memset(pFullPath, 0x00, sizeof(pFullPath));
		strcpy(pFullPath, pPath);
		strcat(pFullPath, pName);
}
/*
 * Create letterbar for the top and bottom of the ASCII board.
 */

static void
make_letterbar(int boardsize, char *letterbar)
{
  int i, letteroffset;
  char spaces[64];
  char letter[64];

  if (boardsize <= 25)
    strcpy(spaces, " ");
  strcpy(letterbar, "   ");
  
  for (i = 0; i < boardsize; i++) {
    letteroffset = 'A';
    if (i+letteroffset >= 'I')
      letteroffset++;
    strcat(letterbar, spaces);
    sprintf(letter, "%c", i+letteroffset);
    strcat(letterbar, letter);
  }
  
}

void DrawLetterbar(int boardsize)
{
	int i;
  char acLetter[2];
  memset(acLetter, 0x00, 2);
  SDL_Rect srMsg={POS_LETTER_X,POS_LETTER_Y,POS_LETTER_W,POS_LETTER_H};
  //background restore - front fuction
	//SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_BOLD);
	srMsg.w=POS_BOARD_SPACE;
	
	for(i = 0; i < boardsize; i++) 
	{
		acLetter[0]='A'+ i;
		if(acLetter[0]>='I')
			acLetter[0]++;
		srMsg.x=POS_LETTER_X+i*POS_BOARD_SPACE;
		sfText = TTF_RenderUTF8_Blended(defaultFont, acLetter, scBlack);
		SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
	}
	//SDL_Flip(sfScreen);
}
/* This array contains +'s and -'s for the empty board positions.
 * hspot_size contains the board size that the grid has been
 * initialized to.
 */

static int hspot_size;
static char hspots[MAX_BOARD][MAX_BOARD];


/*
 * Mark the handicap spots on the board.
 */

static void
set_handicap_spots(int boardsize)
{
  if (hspot_size == boardsize)
    return;
  
  hspot_size = boardsize;
  
  memset(hspots, '.', sizeof(hspots));

  if (boardsize == 5) {
    /* place the outer 4 */
    hspots[1][1] = '+';
    hspots[boardsize-2][1] = '+';
    hspots[1][boardsize-2] = '+';
    hspots[boardsize-2][boardsize-2] = '+';
    /* and the middle one */
    hspots[boardsize/2][boardsize/2] = '+';
    return;
  }

  if (!(boardsize%2)) {
    /* If the board size is even, no center handicap spots. */
    if (boardsize > 2 && boardsize < 12) {
      /* Place the outer 4 only. */
      hspots[2][2] = '+';
      hspots[boardsize-3][2] = '+';
      hspots[2][boardsize-3] = '+';
      hspots[boardsize-3][boardsize-3] = '+';
    }
    else {
      /* Place the outer 4 only. */
      hspots[3][3] = '+';
      hspots[boardsize-4][3] = '+';
      hspots[3][boardsize-4] = '+';
      hspots[boardsize-4][boardsize-4] = '+';
    }
  }
  else {
    /* Uneven board size */
    if (boardsize > 2 && boardsize < 12) {
      /* Place the outer 4... */
      hspots[2][2] = '+';
      hspots[boardsize-3][2] = '+';
      hspots[2][boardsize-3] = '+';
      hspots[boardsize-3][boardsize-3] = '+';

      /* ...and the middle one. */
      hspots[boardsize/2][boardsize/2] = '+';
    }
    else {
      /* Place the outer 4... */
      hspots[3][3] = '+';
      hspots[boardsize-4][3] = '+';
      hspots[3][boardsize-4] = '+';
      hspots[boardsize-4][boardsize-4] = '+';

      /* ...and the inner 4... */
      hspots[3][boardsize/2] = '+';
      hspots[boardsize/2][3] = '+';
      hspots[boardsize/2][boardsize-4] = '+';
      hspots[boardsize-4][boardsize/2] = '+';

      /* ...and the middle one. */
      hspots[boardsize/2][boardsize/2] = '+';
    }
  }

  return;
}

void DrawBoardline(int board_size)
{
	int i,j;
	char acIdx[3];
	memset(acIdx, 0x00, 3);
	SDL_Rect srMsg={POS_BOARD_X,POS_BOARD_Y,272,272};
	SDL_Rect srLine={POS_BOARD_X+25,POS_BOARD_Y+20,POS_BOARD_SPACE*(board_size-1),1};
	//background restore
	SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_BOLD);
	srMsg.x=POS_BOARD_X+2;
	srMsg.y=18;
	srMsg.w=20;
	srMsg.h=POS_BOARD_SPACE;
	srLine.w=POS_BOARD_SPACE*(board_size-1);
	srLine.h=1;
	
	for (i = 0; i < board_size; i++) {
		//Idx
    sprintf(acIdx,"%2d", board_size - i);//index
    sfText = TTF_RenderUTF8_Blended(defaultFont, acIdx, scBlack);
    srLine.y=POS_BOARD_Y+20+i*POS_BOARD_SPACE;
    srMsg.y=srLine.y-6;
    SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
    //horizontal Line
    SDL_FillRect(sfScreen, &srLine, SDL_MapRGB(sfScreen->format, scBlack.r, scBlack.g, scBlack.b));
  }
  srLine.y=POS_BOARD_Y+20;
	srLine.w=1;
	srLine.h=POS_BOARD_SPACE*(board_size-1);
	for (i = 0; i < board_size; i++) {
		//vertical Line
		srLine.x=POS_BOARD_X+25+i*POS_BOARD_SPACE;
    SDL_FillRect(sfScreen, &srLine, SDL_MapRGB(sfScreen->format, scBlack.r, scBlack.g, scBlack.b));
  }
  //ٵġ ׸
  srLine.w=3;
  srLine.h=3;
  for(i=0;i<board_size;i++)
  {
  	for(j=0;j<board_size;j++)
  	{
  		if(hspots[i][j]=='+')
  		{
  			srLine.x=POS_BOARD_X+24+i*POS_BOARD_SPACE;
  			srLine.y=POS_BOARD_Y+19+j*POS_BOARD_SPACE;
  			SDL_FillRect(sfScreen, &srLine, SDL_MapRGB(sfScreen->format, scBlack.r, scBlack.g, scBlack.b));
  		}
  	}
  }
  //SDL_Flip(sfScreen);
}

void DrawStone(int StoneType, int white, int x, int y)
{
	if((showdead==0)&&(StoneType==STONE_DEAD)) return;
	//|缮||쵹
	//|缮||쵹
	SDL_Rect srStone={POS_BOARD_SPACE*StoneType,POS_BOARD_SPACE*white,POS_BOARD_SPACE,POS_BOARD_SPACE};
	SDL_Rect srPos={POS_BOARD_X+19+POS_BOARD_SPACE*y,POS_BOARD_Y+14+POS_BOARD_SPACE*x,POS_BOARD_SPACE,POS_BOARD_SPACE};
	SDL_BlitSurface(sfStone, &srStone,sfScreen, &srPos);
	
}

void DrawInfo(Uint8 nInfoType,Gameinfo *gameinfo)
{
	char acLine[NUM_MAX_LINE];
	char acTemp[10];
	SDL_Rect srMsg={POS_INFO_X,POS_INFO_Y,POS_INFO_W,20};
	SDL_Rect srStone={0,0,13,13};
	memset(acLine,0x00, NUM_MAX_LINE);
	memset(acTemp,0x00,10);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_NORMAL);
	
	switch(nInfoType)
	{
	//: ,,:10
		case INFO_PERSONAL:
		//com
			srMsg.y=3;
			//background restore
			SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
			//com
			if(bRecordSee)//⺸ô  
			{
				sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,black_captured);
			}
			else if(bForce)
			{
				if(bPlayBlack)
					sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,black_captured);
				else
					sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,white_captured);
			}
			else
			{
				sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==gameinfo->computer_player) ? "":"" ,\
												(gameinfo->computer_player==BLACK) ? white_captured : black_captured);
			}
			sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scYellow);
  		SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
  		if(bRecordSee)//⺸ô  
				srStone.y=13;
  		else if(bForce)
  			srStone.y=13*(bPlayBlack ? 1:0);
  		else
				srStone.y=13*((gameinfo->computer_player==BLACK) ? 0:1);
			srMsg.x=POS_INFO_X+40;
			SDL_BlitSurface(sfStone, &srStone, sfScreen, &srMsg);
		//user	
			srMsg.x=POS_INFO_X;
			srMsg.y=255;
			srMsg.w=POS_INFO_W;
			srMsg.h=20;
			//background restore
			SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
			if(bRecordSee)//⺸ô  
			{
				sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,white_captured);
			}
			else if(bForce)
			{
				if(bPlayBlack)
					sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,white_captured);
				else
					sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==BLACK) ? "":"" ,black_captured);
			}
			else
			{
				sprintf(acLine,"%s :    ,: %d",(gameinfo->to_move==gameinfo->computer_player) ? "":"" ,\
												(gameinfo->computer_player==BLACK) ? black_captured : white_captured);
			}
			sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scYellow);
  		SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
  		if(bRecordSee)//⺸ô  
				srStone.y=0;
  		else if(bForce)
  			srStone.y=13*(bPlayBlack ? 0:1);
  		else
				srStone.y=13*((gameinfo->computer_player==BLACK) ? 1:0);
			srMsg.x=POS_INFO_X+40;
			SDL_BlitSurface(sfStone, &srStone, sfScreen, &srMsg);
			break;
	//μ, ,
		case INFO_TOMOVE:
			srMsg.y=POS_INFO_Y+20;
			//background restore
			SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
			if(strcmp(color_to_string(gameinfo->to_move),"black")==0)
				strcpy(acTemp,"");
			else
				strcpy(acTemp,"");
			sprintf(acLine,"%s  (%d)", acTemp, movenum + 1 );//+ gameinfo->handicap ?
			sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scYellow);
  		SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
			break;
	// :  10 ռ
		case INFO_SCORE:
			srMsg.y=POS_INFO_Y+40;
			//background restore
			SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
			if (showscore) { 
		    if (current_score_estimate == NO_SCORE)
		      sprintf(acLine,"   ȵ");
		    else if (current_score_estimate < 0)
		      sprintf(acLine," %d  ռ",
			     -current_score_estimate);
		    else if (current_score_estimate > 0)
		      sprintf(acLine," %d  ռ",
			     current_score_estimate);
		    else
		      sprintf(acLine," ");
		      
		    sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scYellow);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
		  }
			break;
	//ٵ:1 :6.5 :10
		case INFO_OPTION:
			srMsg.y=POS_INFO_Y;
			//background restore
			SDL_BlitSurface(sfBackground, &srMsg, sfScreen, &srMsg);
			sprintf(acLine,"ٵ:%d, :%.1f, :%d",gameinfo->handicap, komi, level);
			sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scYellow);
  		SDL_BlitSurface(sfText, NULL,sfScreen, &srMsg);
			break;
	}
	SDL_Flip(sfScreen);
}
/*
 * Display the board position when playing in ASCII.
 */

static void
ascii_showboard(Gameinfo *gameinfo)
{
  int i, j;
  char letterbar[64];
  int last_pos_was_move;
  int pos_is_move;
  int dead;
  char acLine[80];
	memset(acLine,0x00,80);
#ifdef V43_DEBUG  
  make_letterbar(board_size, letterbar);
#endif
  set_handicap_spots(board_size);

	DrawInfo(INFO_PERSONAL,gameinfo);
	DrawInfo(INFO_SCORE,gameinfo);
  DrawInfo(INFO_OPTION,gameinfo); 
#ifdef V43_DEBUG 
  printf("\n");
  printf("    White (O) has captured %d pieces\n", black_captured);
  printf("    Black (X) has captured %d pieces\n", white_captured);
  
  if (showscore) { 
    if (current_score_estimate == NO_SCORE)
      printf("    No score estimate is available yet.\n");
    else if (current_score_estimate < 0)
      printf("    Estimated score: Black is ahead by %d\n",
	     -current_score_estimate);
    else if (current_score_estimate > 0)
      printf("    Estimated score: White is ahead by %d\n",
	     current_score_estimate);
    else
      printf("    Estimated score: Even!\n");
  }
  
  printf("\n");

  fflush(stdout);
  //printf("%s%s\n", (emacs ? "EMACS1\n" : ""), letterbar);
  printf("%s\n", letterbar);
  fflush(stdout);
#endif
  DrawBoardline(board_size);
  DrawLetterbar(board_size);
  
  for (i = 0; i < board_size; i++) {
#ifdef V43_DEBUG
    printf(" %2d", board_size - i);//index
#endif
    last_pos_was_move = 0;
    for (j = 0; j < board_size; j++) {
      if (last_move_i == i && last_move_j == j)
				pos_is_move = 128;
      else
				pos_is_move = 0;
      dead = (dragon_status(POS(i, j)) == DEAD) && showdead;
      
      switch (BOARD(i, j) + pos_is_move + last_pos_was_move) {
	case EMPTY+128:
	case EMPTY:
#ifdef V43_DEBUG	
	  printf(" %c", hspots[i][j]);
#endif
	  last_pos_was_move = 0;
	  break;
	case BLACK:
		DrawStone(dead,STONE_BLACK,i,j);
#ifdef V43_DEBUG
	  printf(" %c", dead ? 'x' : 'X');
#endif
	  last_pos_was_move = 0;
	  break;
	case WHITE:
		DrawStone(dead,STONE_WHITE,i,j);
#ifdef V43_DEBUG
	  printf(" %c", dead ? 'o' : 'O');
#endif
	  last_pos_was_move = 0;
	  break;
	case BLACK+128://ֽŵ
		DrawStone(STONE_RED,STONE_BLACK,i,j);
		sprintf(acLine," %c%d ",(j+'A'>='I')? 'A'+j+1:'A'+j,board_size-i);
		DrawMessage(acLine,false,0);
#ifdef V43_DEBUG
	  printf("(%c)", 'X');
#endif
	  last_pos_was_move = 256;
	  break;
	case WHITE+128://ֽŵ
		DrawStone(STONE_RED,STONE_WHITE,i,j);
		sprintf(acLine," %c%d ",(j+'A'>='I')? 'A'+j+1:'A'+j,board_size-i);
		DrawMessage(acLine,false,0);
#ifdef V43_DEBUG
	  printf("(%c)", 'O');
#endif
	  last_pos_was_move = 256;
	  break;
	case EMPTY+256:
#ifdef V43_DEBUG
	  printf("%c", hspots[i][j]);
#endif
	  last_pos_was_move = 0;
	  break;
	case BLACK+256:
		DrawStone(dead,STONE_BLACK,i,j);
#ifdef V43_DEBUG
	  printf("%c", dead ? 'x' : 'X');
#endif
	  last_pos_was_move = 0;
	  break;
	case WHITE+256:
		DrawStone(dead,STONE_WHITE,i,j);
#ifdef V43_DEBUG
	  printf("%c", dead ? 'o' : 'O');
#endif
	  last_pos_was_move = 0;
	  break;
	default: 
	  fprintf(stderr, "Illegal board value %d\n", (int) BOARD(i, j));
	  exit(EXIT_FAILURE);
	  break;
      }
    }
#ifdef V43_DEBUG    
    if (last_pos_was_move == 0) {
      if (board_size > 10)
	printf(" %2d", board_size - i);
      else
	printf(" %1d", board_size - i);
    }
    else {
      if (board_size > 10)
	printf("%2d", board_size - i);
      else
	printf("%1d", board_size - i);
    }
    printf("\n");
#endif    
  }
#ifdef V43_DEBUG      
  fflush(stdout);
  printf("%s\n\n", letterbar);
  fflush(stdout);
#endif 
	SDL_Flip(sfScreen);
	
}  /* end ascii_showboard */

/*
 * command help
 */

static void
show_commands(void)
{
  printf("\nCommands:\n");
  printf(" back                Take back your last move\n");
  printf(" boardsize           Set boardsize (on move 1 only)\n");
  printf(" comment             Write a comment to outputfile\n");
  printf(" depth <num>         Set depth for reading\n");
  printf(" display             Display game board\n");
  printf(" exit                Exit GNU Go\n");
  printf(" force <move>        Force a move for current color\n");
  printf(" forward             Go to next node in game tree\n");
  printf(" goto <movenum>      Go to movenum in game tree\n");
  printf(" level <amount>      Playing level (default = 10)\n");
  printf(" handicap <num>      Set fixed handicap (on move 1 only)\n");
  printf(" freehandicap <num>  Place free handicap (on move 1 only)\n");
  printf("                     Omit <num> to place handicap yourself\n");
  printf(" help                Display this help menu\n");
  printf(" helpdebug           Display debug help menu\n");
  printf(" info                Display program settings\n");
  printf(" komi                Set komi (on move 1 only)\n");
  printf(" last                Goto last node in game tree\n");
  printf(" pass                Pass on your move\n");
  printf(" play <num>          Play <num> moves\n");
  printf(" playblack           Play as Black (switch if White)\n");
  printf(" playwhite           Play as White (switch if Black)\n");
  printf(" quit                Exit GNU Go\n");
  printf(" resign              Resign the current game\n");
  printf(" save <file>         Save the current game\n");
  printf(" load <file>         Load a game from file\n");
  printf(" score               Toggle display of score On/Off\n");
  printf(" showboard           Toggle display of board On/Off\n");
  printf(" switch              Switch the color you are playing\n");
  printf(" undo                Take the last move back (same as back)\n");
  printf(" <move>              A move of the format <letter><number>");
  printf("\n");
}

enum commands {INVALID = -1, END, EXIT, QUIT, RESIGN, 
	       PASS, MOVE, FORCE, SWITCH,
	       PLAY, PLAYBLACK, PLAYWHITE,
	       SETHANDICAP, FREEHANDICAP, SETBOARDSIZE, SETKOMI,
	       SETDEPTH,
               INFO, DISPLAY, SHOWBOARD, HELP, UNDO, COMMENT, SCORE,
               CMD_DEAD, CMD_BACK, CMD_FORWARD, CMD_LAST,
               CMD_CAPTURE, CMD_DEFEND,
               CMD_HELPDEBUG, CMD_SHOWAREA, CMD_SHOWMOYO, CMD_SHOWTERRI,
               CMD_GOTO, CMD_SAVE, CMD_LOAD, CMD_SHOWDRAGONS, CMD_LISTDRAGONS,
	       SETHURRY, SETLEVEL, NEW, COUNT, CONTINUE
};

void CopyRect(SDL_Rect *srDest, SDL_Rect *srSrc)
{
	srDest->x=srSrc->x;
  srDest->y=srSrc->y;
  srDest->w=srSrc->w;
  srDest->h=srSrc->h;
}

void ResetRect(SDL_Rect *srSrc)
{
	srSrc->x=0;
  srSrc->y=0;
  srSrc->w=0;
  srSrc->h=0;
}

/*
 * Write sgf root node. 
 */

static void
init_sgf(Gameinfo *ginfo)
{
  if (sgf_initialized)
    return;
  sgf_initialized = 1;

  sgf_write_header(sgftree.root, 1, get_random_seed(), komi,
      		   level, chinese_rules);
  sgfOverwritePropertyInt(sgftree.root, "HA", ginfo->handicap);
  if (ginfo->handicap > 0)
    gnugo_recordboard(sgftree.root);
}

/*
 * Generate the computer move. 
 */

static int
computer_move(Gameinfo *gameinfo, int *passes)
{
  int i, j;
  int move_val;
  int resignation_declined = 0;
  float upper_bound, lower_bound;
	init_sgf(gameinfo);

	DrawInfo(INFO_PERSONAL,gameinfo);
	DrawInfo(INFO_TOMOVE,gameinfo);
  /* Generate computer move. */
  move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
  if (resignation_allowed && move_val < 0.0 && ON_BOARD2(i, j)) {
    int state = ascii_endgame(gameinfo, 2);
    ascii_showboard(gameinfo);
    if (state != -1)
      return state;

    /* The opponent declined resignation. Remember not to resign again. */
    resignation_allowed = 0;
    resignation_declined = 1;
    move_val = -move_val;
  }

  if (showscore) {
    gnugo_estimate_score(&upper_bound, &lower_bound);
    current_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
  }
    
  last_move_i = i;
  last_move_j = j;
  
 // mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move),movenum+1, i, j);
	//DrawInfo(INFO_TOMOVE,gameinfo);
	
  if (is_pass(POS(i, j)))
  {
    (*passes)++;
    DrawMessage(" н߽ϴ.",true,0);
  }
  //else
  //  *passes = 0;

  gnugo_play_move(i, j, gameinfo->to_move);
  sgffile_add_debuginfo(sgftree.lastnode, move_val);
  sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
  if (resignation_declined)
    sgftreeAddComment(&sgftree, "Baduk resignation was declined");
  sgffile_output(&sgftree);

	if(bSound)
	{
		PlayWav(mcStone,1); 
		SDL_Delay(500);
	}
  gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
  return 0;
}

void DrawOptionMenu(Gameinfo *gameinfo)
{
	char acLine[NUM_MAX_LINE];
	SDL_Rect srBack={POS_BOARD_X,POS_BOARD_Y,272,272};
	SDL_Rect srLine={POS_BOARD_X+50,POS_BOARD_Y+20,POS_BOARD_SIZE-20,20};
	SDL_Rect srIcon={0,0,16,16};
	SDL_Rect srIconPos={POS_BOARD_X+150,POS_BOARD_Y+20,16,16};
	memset(acLine, 0x00, NUM_MAX_LINE);
	
	//background restore
	SDL_BlitSurface(sfBackground, &srBack, sfScreen, &srBack);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_NORMAL);
	//title
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(szVerInfo), scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //sound - option box
  srLine.x=srLine.x-30;
	srLine.y=srLine.y+30;
	strcpy(acLine,"      :    ON,    OFF");
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //option icon
  srIconPos.y=srLine.y;
  srIcon.x=16*ICON_OPTIONBOX_IDX;
  srIcon.y=16*(bSound);
  srIconPos.x=POS_BOARD_X+100;
  CopyRect(&srOption[0][OPT_SOUND_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.y=16*(!bSound);
  srIconPos.x=POS_BOARD_X+150;
  CopyRect(&srOption[1][OPT_SOUND_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  //timer
  srLine.y=srLine.y+20;
	sprintf(acLine," Ÿ̸():     %2d",iSecond);//1~60
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //arrow icon
  srIconPos.y=srLine.y;
  srIcon.x=16*((iSecond<=1)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=0;
  srIconPos.x=POS_BOARD_X+100;
  //button ǥ init
  CopyRect(&srOption[0][OPT_TIMER_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.x=16*((iSecond>=60)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=16;
  srIconPos.x=POS_BOARD_X+150;
  //button ǥ init
  CopyRect(&srOption[1][OPT_TIMER_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  
	//boardsize - left,right arrow (5~19)  <- 19 ->
	srLine.y=srLine.y+20;
	sprintf(acLine,"ٵ ũ:     %2d",iBoardSize);
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //arrow icon
  srIconPos.y=srLine.y;
  srIcon.x=16*((iBoardSize<=5)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=0;
  srIconPos.x=POS_BOARD_X+100;
  //button ǥ init
  CopyRect(&srOption[0][OPT_BOARDSIZE_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.x=16*((iBoardSize>=19)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=16;
  srIconPos.x=POS_BOARD_X+150;
  //button ǥ init
  CopyRect(&srOption[1][OPT_BOARDSIZE_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  
	//level - left,right arrow (1~10)
	srLine.y=srLine.y+20;
	sprintf(acLine,"   :     %2d",iLevel);
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //arrow icon
  srIconPos.y=srLine.y;
  srIcon.x=16*(iLevel ? ICON_LRARROW_IDX : ICON_DISABLELR_IDX);
  srIcon.y=0;
  srIconPos.x=POS_BOARD_X+100;
  CopyRect(&srOption[0][OPT_LEVEL_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.x=16*((iLevel>=10)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=16;
  srIconPos.x=POS_BOARD_X+150;
  CopyRect(&srOption[1][OPT_LEVEL_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  
	//ٵ - left,right arrow, 0 ~ 9
	srLine.y=srLine.y+20;
	sprintf(acLine,"     :     %2d",iHandiCap);
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //arrow icon
  srIconPos.y=srLine.y;
  srIcon.x=16*(iHandiCap ? ICON_LRARROW_IDX : ICON_DISABLELR_IDX);
  srIcon.y=0;
  srIconPos.x=POS_BOARD_X+100;
  CopyRect(&srOption[0][OPT_HANDICAP_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.x=16*((iHandiCap>=9)? ICON_DISABLELR_IDX:ICON_LRARROW_IDX);
  srIcon.y=16;
  srIconPos.x=POS_BOARD_X+150;
  CopyRect(&srOption[1][OPT_HANDICAP_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  
	// - option box, 0~ 6.5 (baduk_config.ini  , ٵ  0)
	srLine.y=srLine.y+20;
	sprintf(acLine,"         :      0.0     %.1f",fSetKomi);
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), iHandiCap ? scGray : scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
	//option icon
	srIconPos.y=srLine.y;
	srIcon.x=16*(iHandiCap ? ICON_DISABLEOPT_IDX : ICON_OPTIONBOX_IDX) ;
	srIcon.y=16*(!bSetKomi);
	srIconPos.x=POS_BOARD_X+100;
	CopyRect(&srOption[0][OPT_ADVANTAGE_IDX],&srIconPos);
	SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
	srIcon.y=16*bSetKomi;
	srIconPos.x=POS_BOARD_X+150;
	CopyRect(&srOption[1][OPT_ADVANTAGE_IDX],&srIconPos);
	SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);

	// - ,  - option box
	srLine.y=srLine.y+20;
	strcpy(acLine,"     :     ,     ");
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //option icon
  srIconPos.y=srLine.y;
  srIcon.x=16*ICON_OPTIONBOX_IDX;
  srIcon.y=16*(bPlayBlack);
  srIconPos.x=POS_BOARD_X+100;
  CopyRect(&srOption[0][OPT_STONE_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  srIcon.y=16*(!bPlayBlack);
  srIconPos.x=POS_BOARD_X+150;
  CopyRect(&srOption[1][OPT_STONE_IDX],&srIconPos);
  SDL_BlitSurface(sfIcon, &srIcon,sfScreen, &srIconPos);
  
	//,,뱹(solo) button
	srIconPos.y=srLine.y+30;
  srIcon.x=0;
  srIcon.y=0;
  srIcon.w=POS_BUTTON_W;
  srIcon.h=POS_BUTTON_H;
  srIconPos.w=srIcon.w;
  srIconPos.h=srIcon.h;
  srIconPos.x=POS_BOARD_X+50;
  //button ǥ init
  CopyRect(&srButton[0],&srIconPos);
  SDL_BlitSurface(sfButton, &srIcon,sfScreen, &srIconPos);
  strcpy(acLine,"뱹 ");
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  srLine.x=srIconPos.x+10;
  srLine.y=srIconPos.y+3;
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  
  srIconPos.x=srIconPos.x+POS_BUTTON_W+10;
  CopyRect(&srButton[1],&srIconPos);
  SDL_BlitSurface(sfButton, &srIcon,sfScreen, &srIconPos);
  strcpy(acLine,"⺸ ۼ");
  sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  srLine.x=srIconPos.x+10;
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  /*
  srIconPos.x=srIconPos.x+POS_BUTTON_W+10;
  CopyRect(&srButton[2],&srIconPos);
  SDL_BlitSurface(sfButton, &srIcon,sfScreen, &srIconPos);
  strcpy(acLine," ϱ");
  sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  srLine.x=srIconPos.x+10;
  SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  */
  SDL_Flip(sfScreen);
}

void CheckOptionMenu(Gameinfo *gameinfo,int *state,int *passes)
{
	int i,j;
	bool bRedraw;
	//init
	//iBoardSize=board_size;
	//iHandiCap=gameinfo->handicap;
	//iLevel=level;
	
	DrawOptionMenu(gameinfo);
	
  while(1)
  {
	  while ( SDL_PollEvent(&event) ) 
		{
			if(event.type==SDL_MOUSEBUTTONDOWN)
			{
				SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_IGNORE);
			 	SDL_EventState(SDL_MOUSEMOTION,SDL_IGNORE);
			 	SDL_EventState(SDL_KEYDOWN,SDL_IGNORE);
			 	bRedraw=false;
				//icon click
				for(i=0;i<2;i++)
				{
					for(j=0;j<NUM_OPTION_CNT;j++)
					{
						if((srOption[i][j].x < event.button.x) && (srOption[i][j].x+srOption[i][j].w > event.button.x) \
				 		&& (srOption[i][j].y < event.button.y) && (srOption[i][j].y+srOption[i][j].h > event.button.y))
						{
							if(i==0) //left
							{
								switch(j)
								{
									case OPT_SOUND_IDX			:
										if(bSound==false)
										{
											bSound=true;
											bRedraw=true;
										}
										break;
									case OPT_TIMER_IDX			:
										if(iSecond>1)
										{
											iSecond--;
											bRedraw=true;
										}
										break;
									case OPT_LEVEL_IDX			: 
										if(iLevel>0) 
										{
											iLevel--; 
											bRedraw=true;
										}
										break;
									case OPT_HANDICAP_IDX	  : 
										if(iHandiCap>0) 
										{
											iHandiCap--;
											if(iHandiCap>0)
											{
												//komi=0.0;
												bSetKomi=false;
											}
											else 
											{
												bSetKomi=true;
											}
											bRedraw=true;
										}
										break;
									case OPT_ADVANTAGE_IDX	: 
										if(iHandiCap==0)
										{
											//komi=0.0; 
											bSetKomi=false;
											bRedraw=true;
										}
										break;
									case OPT_BOARDSIZE_IDX	: 
										if(iBoardSize>5) 
										{
											iBoardSize--; 
											bRedraw=true;
										}
										break;
									case OPT_STONE_IDX			: 
										if(bPlayBlack==false)
										{
											bPlayBlack=true;
											bRedraw=true;
										}
										break;
								}
								
							}
							else if(i==1) //right
							{
								switch(j)
								{
									case OPT_SOUND_IDX			:
										if(bSound==true)
										{
											bSound=false;
											bRedraw=true;
										}
										break;
									case OPT_TIMER_IDX			:
										if(iSecond<60)
										{
											iSecond++;
											bRedraw=true;
										}
										break;
									case OPT_LEVEL_IDX			: 
										if(iLevel<10)
										{ 
											iLevel++; 
											bRedraw=true;
										}
										break;
									case OPT_HANDICAP_IDX	  : 
										if(iHandiCap<9)
										{ 
											iHandiCap++;
											//komi=0.0;
											bSetKomi=false;
											bRedraw=true;
										}	
										break;
									case OPT_ADVANTAGE_IDX	: 
										if(iHandiCap==0)
										{ 
											//komi=fSetKomi; 
											bSetKomi=true;
											bRedraw=true;
										}
										break;
									case OPT_BOARDSIZE_IDX	: 
										if(iBoardSize<19)
										{ 
											iBoardSize++;
											bRedraw=true; 
										}
										break;
									case OPT_STONE_IDX			: 
										if(bPlayBlack==true)
										{
											bPlayBlack=false;
											bRedraw=true; 
										}
										break;
								}
							}
							if(bRedraw)
								DrawOptionMenu(gameinfo);
							//printf("icon click[%d][%d]\n",i,j);
						}
					}
				}
				//button click
			 	for(i=0;i<2;i++)
				{
					if((srButton[i].x < event.button.x) && (srButton[i].x+srButton[i].w > event.button.x) \
					&& (srButton[i].y < event.button.y) && (srButton[i].y+srButton[i].h > event.button.y))
					{
							if(i==0)//play
							{
								if(bSetKomi) //(iHandiCap==0)
									komi=fSetKomi;
								else
									komi=0.0;
								
								level=iLevel;
								/* Init board. */
								gnugo_clear_board(iBoardSize);
							  /* In case max handicap changes on smaller board. */
							  gameinfo->handicap = gnugo_placehand(iHandiCap);
							  gameinfo->to_move = (gameinfo->handicap ? WHITE : BLACK);
							  sgfOverwritePropertyInt(sgftree.root, "SZ", iBoardSize);
							  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
							  bForce=false;
							  bRecordSee=false;
							  if(bPlayBlack)
								{
									if(gameinfo->computer_player == BLACK)
									  gameinfo->computer_player = WHITE;
									//if (gameinfo->computer_player == gameinfo->to_move)
									  //*state = computer_move(gameinfo, passes);
								}
								else
								{
									if(gameinfo->computer_player == WHITE)
									    gameinfo->computer_player = BLACK;
									//if(gameinfo->computer_player == gameinfo->to_move)
									    //*state = computer_move(gameinfo, passes);
								}
								if(gameinfo->handicap>0) 
									  gameinfo->to_move = WHITE;
								else
										gameinfo->to_move = BLACK;
										
								SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
								SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
								SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
								*state = 0;//continue
								return ;
							}
							else  //⺸ ۼ
							{
								if(bSetKomi) //(iHandiCap==0)
									komi=fSetKomi;
								else
									komi=0.0;
								
								level=iLevel;
								/* Init board. */
								gnugo_clear_board(iBoardSize);
							  /* In case max handicap changes on smaller board. */
							  gameinfo->handicap = gnugo_placehand(iHandiCap);
							  gameinfo->to_move = (gameinfo->handicap ? WHITE : BLACK);
							  sgfOverwritePropertyInt(sgftree.root, "SZ", iBoardSize);
							  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
							  bForce=true;
							  bRecordSee=false;
							  
								if(gameinfo->handicap>0) 
									gameinfo->to_move = WHITE;
								else
									gameinfo->to_move = BLACK;
								
							  SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
								SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
								SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
								*state = 0;//continue
								return;
							}
							/*
							else //quit
							{
								SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
								SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
								SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
								*state = 2;
								return ;
							}
							*/
							//printf("button click[%d]\n",i);
					}
					
				}
			}
		}
		SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
		SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
		SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
		SDL_Delay(100);  
	}

}

void SwapData(FILELIST* pfl, int a, int b)
{
	FILEDATA stTmpData[1];
	
	stTmpData[0].bFolder = pfl->stFileData[b].bFolder;
	strcpy(stTmpData[0].acFileName, pfl->stFileData[b].acFileName);
	
	pfl->stFileData[b].bFolder = pfl->stFileData[a].bFolder;
	strcpy(pfl->stFileData[b].acFileName, pfl->stFileData[a].acFileName);
	
	pfl->stFileData[a].bFolder = stTmpData[0].bFolder;
	strcpy(pfl->stFileData[a].acFileName, stTmpData[0].acFileName);
}

void SortData(FILELIST* pfl, bool bSortAsc)
{
	int i,j;
	
	for( i=1;i<(pfl->nCount-1);i++) //  
	{
		for( j=i+1;j<(pfl->nCount);j++)
		{
			if(bSortAsc)
			{
				if((pfl->stFileData[i].bFolder < pfl->stFileData[j].bFolder) )
				{
					SwapData(pfl, i, j);
				}
			}
			else
			{
				if((pfl->stFileData[i].bFolder > pfl->stFileData[j].bFolder) )
				{
					SwapData(pfl, i, j);
				}
			}
		}
	}
	for( i=1;i<(pfl->nCount-1);i++) // Ʈ 
	{
		for( j=i+1;j<(pfl->nCount);j++)
		{
			if(bSortAsc)
			{
				if((pfl->stFileData[i].bFolder == pfl->stFileData[j].bFolder) && (strcmp(pfl->stFileData[i].acFileName, pfl->stFileData[j].acFileName)>0) )
				{
					SwapData(pfl, i, j);
				}
			}
			else
			{
				if((pfl->stFileData[i].bFolder == pfl->stFileData[j].bFolder) && (strcmp(pfl->stFileData[i].acFileName, pfl->stFileData[j].acFileName)<0) )
				{
					SwapData(pfl, i, j);
				}
			}
		}
	}
}

int GenerateFileList (FILELIST* pfl, char *szFilter, bool bSortAsc) // Find szFilter extension in the folder
{
	DIR *dp;
	struct dirent *dirp;
	int i;
	char *pFind;
		
	pfl->nCount = 0;
	pfl->nCurPos = 0;
	pfl->nPagePos = 0;
	
	for(i=0;i<NUM_MAX_FILE;i++) //̵ ܻ
	{
		pfl->stFileData[i].bFolder=false;
		memset(pfl->stFileData[i].acFileName,0x00,NUM_MAX_LINE);
	}
	
	i = strlen(pfl->acCurDir);
	if (pfl->acCurDir[i-1] == '/')
		pfl->acCurDir[i-1] = '\0';	// remove slash
		
	if((dp = opendir(pfl->acCurDir)) == NULL)
	{
		printf("Opendir failed[%s]\n",pfl->acCurDir);
		return 0;
	}
	if(pfl->nCount==0) // ߰
	{
		pfl->stFileData[pfl->nCount].bFolder=true;
		strcpy(pfl->stFileData[pfl->nCount].acFileName, "..");
		pfl->nCount++;
	}
	
	while((dirp = readdir(dp))!=NULL)
	{
		//printf("d_ino:%d \td_type:%d \ti_name:%s\n", dirp->d_ino,dirp->d_type,dirp->d_name); //d_type : 4 , 8 , 10 ũ

		if(dirp->d_name[0] == '.')// '.'  ϴ  
			continue;
			
		if(dirp->d_type==4) //  ߰
		{
			pfl->stFileData[pfl->nCount].bFolder=true;
			strncpy(pfl->stFileData[pfl->nCount].acFileName, dirp->d_name,NUM_MAX_LINE);
			pfl->nCount++;
			continue;
		}
		
		pFind = NULL;
		pFind = strstr(dirp->d_name, szFilter); 
		
		if ((pFind != NULL) || (strlen(szFilter)==0)) //͸ ɷȰų ͼ ȵǾ   üũ 
		{
			if(dirp->d_type==8) //file
				pfl->stFileData[pfl->nCount].bFolder=false;
			else
				continue;
			strncpy(pfl->stFileData[pfl->nCount].acFileName, dirp->d_name, NUM_MAX_LINE);
			pfl->nCount++;
			if (pfl->nCount >= NUM_MAX_FILE) break;
		}

	}
	closedir(dp);
	
	//list sort
	SortData(pfl, bSortAsc);
	
	return pfl->nCount;
}

void DrawFileName(FILELIST *pfl, bool bSelected, SDL_Rect srFileName, int nCurPos)
{	
	char acLine[NUM_MAX_LINE];
	memset(acLine, 0x00, NUM_MAX_LINE);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_NORMAL);
	strncpy(acLine,pfl->stFileData[nCurPos].acFileName,NUM_COPY_LINE);
	if(bSelected)
	{
		SDL_FillRect(sfScreen, &srFileName, SDL_MapRGB(sfScreen->format, scGreen.r, scGreen.g, scGreen.b));
		sfText = TTF_RenderUTF8_Blended(defaultFont, acLine, scWhite);
	}
	else
	{
		SDL_FillRect(sfScreen, &srFileName, SDL_MapRGB(sfScreen->format, scWhite.r, scWhite.g, scWhite.b));
		sfText = TTF_RenderUTF8_Blended(defaultFont, acLine, scBlack);
	}
	SDL_BlitSurface(sfText, NULL, sfScreen, &srFileName);
}

SDL_Rect DrawScrollBar(FILELIST* pfl)
{
	//Scroll range : 168, 1 line height : 20, 1 scroll icon : 16
	int nThumbHeight=0;
	double fTemp;
	if(pfl->nCount >= NUM_MAX_PAGE)
	{
		fTemp = 168 * NUM_MAX_PAGE;//200-32
		fTemp = (fTemp / (pfl->nCount));
		nThumbHeight = (int)floor(fTemp);
	}
	
	int nThumbPos = POS_LOADBOX_Y+16; // + scroll icon
	
	if((pfl->nCount-(pfl->nPagePos*NUM_MAX_PAGE))<NUM_MAX_PAGE) //last page position
	{
		nThumbPos = POS_LOADBOX_Y + 20*NUM_MAX_PAGE - nThumbHeight - 16;
	}
	else 
	{
		nThumbPos = nThumbPos + (nThumbHeight * pfl->nPagePos);
	}
	//Draw Scrollbar
	SDL_Rect srScrollbar={POS_LOADBOX_X+POS_LOADBOX_W-16,POS_LOADBOX_Y,16,POS_LOADBOX_H};
	SDL_FillRect(sfScreen, &srScrollbar, SDL_MapRGB(sfScreen->format, scGray.r, scGray.g, scGray.b));
	
	//Draw Thumb
	SDL_Rect srScrollThumb={POS_LOADBOX_X+POS_LOADBOX_W-16,nThumbPos,16,nThumbHeight};
	SDL_FillRect(sfScreen, &srScrollThumb, SDL_MapRGB(sfScreen->format, scGreen.r, scGreen.g, scGreen.b));
	//button ǥ 
	srOption[0][LOAD_PAGEUPDOWN_IDX].x=srScrollbar.x;
	srOption[0][LOAD_PAGEUPDOWN_IDX].y=POS_LOADBOX_Y+16;
	srOption[0][LOAD_PAGEUPDOWN_IDX].w=16;
	srOption[0][LOAD_PAGEUPDOWN_IDX].h=nThumbPos-srOption[0][LOAD_PAGEUPDOWN_IDX].y;
	
	srOption[1][LOAD_PAGEUPDOWN_IDX].x=srScrollbar.x;
	srOption[1][LOAD_PAGEUPDOWN_IDX].y=nThumbPos+nThumbHeight;
	srOption[1][LOAD_PAGEUPDOWN_IDX].w=16;
	srOption[1][LOAD_PAGEUPDOWN_IDX].h=POS_LOADBOX_Y+POS_LOADBOX_H-16-srOption[1][LOAD_PAGEUPDOWN_IDX].y;
	//printf("height:%d,pos:%d,cnt:%d,cur:%d,range:%d,page:%d\n",nThumbHeight,nThumbPos,pfl->nCount,pfl->nCurPos,nMoveRange,nPagePos);
	return srScrollThumb;
}

void DrawLoadList()
{
	int i;
	char acLine[NUM_MAX_LINE];
	SDL_Rect srBack={POS_BOARD_X,POS_BOARD_Y,272,272};
	SDL_Rect srLine={POS_LOADLINE_X,POS_LOADLINE_Y,POS_LOADLINE_W,POS_LOADLINE_H};
	SDL_Rect srBox={POS_LOADBOX_X,POS_LOADBOX_Y,POS_LOADBOX_W,POS_LOADBOX_H};
	SDL_Rect srIcon={16*ICON_FOLDERFILE_IDX,0,16,16};
	memset(acLine,0x00, NUM_MAX_LINE);
	
	//background restore
	SDL_BlitSurface(sfBackground, &srBack, sfScreen, &srBack);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_NORMAL);
	//folder icon
	SDL_BlitSurface(sfIcon, &srIcon, sfScreen, &srLine);
	//Default path print
	srLine.x=POS_LOADLINE_X+20;
	strncpy( acLine, stFileList.acCurDir,NUM_COPY_LINE);// ȳġ
	sfText = TTF_RenderUTF8_Blended(defaultFont, acLine, scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
	
	//file count
	srLine.x=POS_LOADLINE_X;
  srLine.y=POS_LOADLINE_Y+20;
  sprintf(acLine,"Count:%d",stFileList.nCount);
  sfText = TTF_RenderUTF8_Blended(defaultFont, acLine, scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
  //,  ư
  srLine.x=POS_LOADLINE_X+POS_LOADLINE_W-170;
  //srLine.y=POS_LOADLINE_Y+20;
  srLine.w=POS_BUTTON_W;
  srLine.h=POS_BUTTON_H;
  SDL_BlitSurface(sfButton, NULL,sfScreen, &srLine);
  CopyRect(&srOption[0][LOAD_LOADFILE_IDX],&srLine);
  srLine.x=srLine.x+13;
  srLine.y=srLine.y+3;
  sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf("̾α"), scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
	srLine.x=POS_LOADLINE_X+POS_LOADLINE_W-80;
	srLine.y=POS_LOADLINE_Y+20;
	SDL_BlitSurface(sfButton, NULL,sfScreen, &srLine);
	CopyRect(&srOption[1][LOAD_LOADFILE_IDX],&srLine);
	srLine.x=srLine.x+13;
  srLine.y=srLine.y+3;
	sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf("⺸"), scBlack);
	SDL_BlitSurface(sfText, NULL,sfScreen, &srLine);
	
	//ϸƮ,ũ
	SDL_FillRect(sfScreen, &srBox, SDL_MapRGB(sfScreen->format, scWhite.r, scWhite.g, scWhite.b));
	//ϸ  ׸
	for(i=0;i<NUM_MAX_PAGE;i++)
	{
		if(strlen(stFileList.stFileData[stFileList.nPagePos*NUM_MAX_PAGE+i].acFileName)==0)
			break;	
		
		srLine.x = POS_LOADBOX_X+20;
		srLine.y = POS_LOADBOX_Y+20*i+1;
		srLine.w = POS_LOADBOX_W - ((stFileList.nCount>NUM_MAX_PAGE) ? 36:20);
		srLine.h = 20;
		if(((i+stFileList.nPagePos*NUM_MAX_PAGE)==stFileList.nCurPos)&&(stFileList.stFileData[stFileList.nCurPos].bFolder==false)) //õ ϸ ׸
			DrawFileName(&stFileList, true, srLine, stFileList.nPagePos*NUM_MAX_PAGE+i);
		else
			DrawFileName(&stFileList, false, srLine, stFileList.nPagePos*NUM_MAX_PAGE+i);
		
		//folder,file icon draw
		srIcon.y = 16*(!stFileList.stFileData[stFileList.nPagePos*NUM_MAX_PAGE+i].bFolder);
		srLine.x = POS_LOADBOX_X;
		srLine.y = POS_LOADBOX_Y+20*i;
		SDL_BlitSurface(sfIcon, &srIcon, sfScreen, &srLine);
	}
	
	//draw scrollbar
	if(stFileList.nCount > NUM_MAX_PAGE)
	{
		srThumb = DrawScrollBar(&stFileList); //for check out size of scrollthumb
		srOption[0][LOAD_ONEUPDOWN_IDX].x=POS_LOADBOX_X+POS_LOADBOX_W-16;
		srOption[0][LOAD_ONEUPDOWN_IDX].y=POS_LOADBOX_Y;
		srOption[0][LOAD_ONEUPDOWN_IDX].w=16;
		srOption[0][LOAD_ONEUPDOWN_IDX].h=16;
		srOption[1][LOAD_ONEUPDOWN_IDX].x=srOption[0][LOAD_ONEUPDOWN_IDX].x;
		srOption[1][LOAD_ONEUPDOWN_IDX].y=POS_LOADBOX_Y+20*NUM_MAX_PAGE-16;
		srOption[1][LOAD_ONEUPDOWN_IDX].w=16;
		srOption[1][LOAD_ONEUPDOWN_IDX].h=16;
		//arrow icon
		srIcon.x=16*ICON_UDARROW_IDX;
		srIcon.y=0;
		SDL_BlitSurface(sfIcon, &srIcon, sfScreen, &srOption[0][LOAD_ONEUPDOWN_IDX]);
		srIcon.y=16;
		SDL_BlitSurface(sfIcon, &srIcon, sfScreen, &srOption[1][LOAD_ONEUPDOWN_IDX]);
	}
	else
	{
		for(i=0;i<2;i++)
		{
			ResetRect(&srOption[i][LOAD_ONEUPDOWN_IDX]);
			ResetRect(&srOption[i][LOAD_PAGEUPDOWN_IDX]);
		}
	}
	
	SDL_Flip(sfScreen); //Refresh
}

void SetNewPathName (FILELIST* pfl)
{
	char acNewPath[NUM_MAX_LINE];
	memset(acNewPath,0x00,NUM_MAX_LINE);
	if((pfl->nCurPos!=0) && (pfl->stFileData[pfl->nCurPos].bFolder)) //  ̵
	{
		sprintf(acNewPath,"%s/%s",pfl->acCurDir,pfl->stFileData[pfl->nCurPos].acFileName);
		strcpy(pfl->acCurDir,acNewPath);
	}
	else if(pfl->nCurPos==0 && (strcmp(pfl->stFileData[pfl->nCurPos].acFileName,"..")==0)) //  ̵
	{
		// ̵ 
#ifdef V43		
		if(strcmp(pfl->acCurDir,"/mnt/hd")==0) return; 
#else
		if(strcmp(pfl->acCurDir,"/home/v43")==0) return; 
#endif	
		int ch = '/';
		char *pUpFolder;
		pUpFolder = strrchr( pfl->acCurDir, ch );
		pfl->acCurDir[pUpFolder - pfl->acCurDir] ='\0';
	}
	
}

int CheckLoadMenu(char *szCmd)
{
	memset(stFileList.acCurDir,0x00,NUM_MAX_LINE);
	strcpy(stFileList.acCurDir,szSGFPath);
	
	GenerateFileList(&stFileList,STR_FILTER,true);
	DrawLoadList();

	int i,j;
	bool bRedraw;
	
	while(1)
	{
		while ( SDL_PollEvent(&event) )
		{
			if(event.type==SDL_MOUSEBUTTONDOWN)
			{
				SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_IGNORE);
			 	SDL_EventState(SDL_MOUSEMOTION,SDL_IGNORE);
			 	SDL_EventState(SDL_KEYDOWN,SDL_IGNORE);
			 	bRedraw=false;
			 	//scroll,arrow click
			 	for(i=0;i<2;i++)
				{
					for(j=0;j<3;j++)
					{
						if((srOption[i][j].x < event.button.x) && (srOption[i][j].x+srOption[i][j].w > event.button.x) \
				 		&& (srOption[i][j].y < event.button.y) && (srOption[i][j].y+srOption[i][j].h > event.button.y))
						{
							switch(j)
							{
								case LOAD_ONEUPDOWN_IDX :
									if(i==0)//up
									{
										if(stFileList.nCurPos>1)
										{
											stFileList.nCurPos--;
											if(stFileList.nCurPos<(stFileList.nPagePos*NUM_MAX_PAGE))
												stFileList.nPagePos--;
											bRedraw=true;
										}
									}
									else //down
									{
										if(stFileList.nCurPos<(stFileList.nCount-1))
										{
											stFileList.nCurPos++;
											if(stFileList.nCurPos>(stFileList.nPagePos*NUM_MAX_PAGE+NUM_MAX_PAGE-1))
												stFileList.nPagePos++;
											bRedraw=true;
										}
									}
									break;
								case LOAD_PAGEUPDOWN_IDX :
									if(i==0)//up
									{
										if(stFileList.nPagePos>0)
										{
											stFileList.nPagePos--;
											stFileList.nCurPos=stFileList.nPagePos*NUM_MAX_PAGE+NUM_MAX_PAGE-1;
											bRedraw=true;
										}
									}
									else //down
									{
										if((stFileList.nPagePos*NUM_MAX_PAGE)<stFileList.nCount)
										{
											stFileList.nPagePos++;
											stFileList.nCurPos=stFileList.nPagePos*NUM_MAX_PAGE;
											bRedraw=true;
										}
									}
									break;
								case LOAD_LOADFILE_IDX :
									if(i==0)//̾α
									{
										if(stFileList.stFileData[stFileList.nCurPos].bFolder==true)
											break;
										sprintf(szCmd,"load %s/%s",stFileList.acCurDir,stFileList.stFileData[stFileList.nCurPos].acFileName);
										bRecordSee=false;
										bForce=false;
										return CMD_LOAD;
									}
									else //⺸
									{
										//return DISPLAY;
										if(stFileList.stFileData[stFileList.nCurPos].bFolder==true)
											break;
										sprintf(szCmd,"load %s/%s",stFileList.acCurDir,stFileList.stFileData[stFileList.nCurPos].acFileName);
										bRecordSee=true;
										bForce=true;
										return CMD_LOAD;
									}
									break;
							}
							
						}
						
					}
				}
				//line click
				for(i=0;i<NUM_MAX_PAGE;i++)
				{
					if((POS_LOADBOX_X < event.button.x) && (POS_LOADBOX_X+POS_LOADBOX_W-16 > event.button.x) \
				 		&& (POS_LOADBOX_Y+i*20 < event.button.y) && (POS_LOADBOX_Y+i*20+20 > event.button.y))
				 	{
				 		stFileList.nCurPos=NUM_MAX_PAGE*stFileList.nPagePos+i;
				 		if(stFileList.stFileData[stFileList.nCurPos].bFolder)
				 		{
							SetNewPathName(&stFileList);
							GenerateFileList(&stFileList,STR_FILTER,true);
						}
						bRedraw=true;
					}
				}
			 	if(bRedraw)
					DrawLoadList();
			}
			
		}
		SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
		SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
		SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
		SDL_Delay(100);  
	}
}

void SaveSGFFile(char *szCmd)
{
	//get datetime
	char acFileName[NUM_MAX_LINE];
	char acLine[NUM_MAX_LINE];
	struct      tm *save_time; //۽ð
	time_t      long_time; /* ð Temp    */
	time(&long_time);
 	save_time = localtime(&long_time);
 	
	memset(acFileName, 0x00, NUM_MAX_LINE);
	memset(acLine, 0x00, NUM_MAX_LINE);
	sprintf(acFileName,"baduk%d_%04d%02d%02d_%02d%02d%02d.sgf",movenum,save_time->tm_year+1900,save_time->tm_mon+1,save_time->tm_mday,\
											save_time->tm_hour, save_time->tm_min, save_time->tm_sec);
	MakeFullFilePath(szSGFPath,acFileName,acFullPath);
	
	sprintf(szCmd,"save %s",acFullPath);
	
	sprintf(acLine,"%s\n Ǿϴ.",acFileName);
	DrawMessage(acLine,true,1000);
	
}

void DrawButton()
{
/*
#define BTN_BACK_IDX 		0
#define BTN_PASS_IDX		1
#define BTN_SCORE_IDX		2
#define BTN_DEAD_IDX		3
#define BTN_MOYO_IDX		4
#define BTN_RESIGN_IDX	5
#define BTN_LOAD_IDX		6
#define BTN_SAVE_IDX		7
#define BTN_OPTION_IDX  8
#define BTN_QUIT_IDX		9

#define BTN_REC_SCORE_IDX		0
#define BTN_REC_LOAD_IDX		1
#define BTN_REC_LONE_IDX		2
#define BTN_REC_RONE_IDX		3
#define BTN_REC_LTEN_IDX  	4
#define BTN_REC_RTEN_IDX		5
#define BTN_REC_FIRST_IDX		6
#define BTN_REC_LAST_IDX		7
#define BTN_REC_OPTION_IDX  8
#define BTN_REC_QUIT_IDX		9
*/
	int i,nLine=0;
	char acLine[16];
	SDL_Rect srBtn={POS_BUTTON_X,POS_BUTTON_Y,POS_BUTTON_W,POS_BUTTON_H};
	memset(acLine,0x00,16);
	TTF_SetFontStyle(defaultFont, TTF_STYLE_NORMAL);
	
	for(i=0;i<NUM_BUTTON_CNT;i++)
	{
		if((i+1)%2==0) //even
		{
			srBtn.x=POS_BUTTON_X+POS_BTNNEXT_X;
			srBtn.y=srBtn.y-3;
			nLine++;
		}
		else //odd
		{
			srBtn.x=POS_BUTTON_X;
			srBtn.y=POS_BUTTON_Y+POS_BTNNEXT_Y*nLine;
		}
		//button	
		SDL_BlitSurface(sfButton, NULL,sfScreen, &srBtn);
		//button ǥ init
		srButton[i].x=srBtn.x;
		srButton[i].y=srBtn.y;
		srButton[i].w=srBtn.w;
		srButton[i].h=srBtn.h;
		
		srBtn.x=srBtn.x+10;
		srBtn.y=srBtn.y+3;//odd
		
		switch(i)
		{
			case BTN_BACK_IDX:
				if(bRecordSee)
				{
					strcpy(acLine,"ڵ ");
					sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), bTimer ? scYellow : scBlack);
				}
				else
				{
					strcpy(acLine," ");
					sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
				}
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_PASS_IDX:
				if(bRecordSee)
					strcpy(acLine,"ҷ ");
				else
					strcpy(acLine,"     ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_SCORE_IDX:
				if(bRecordSee)
				{
					strcpy(acLine,"  (ĭ)");
					sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
				}
				else
				{
					strcpy(acLine," ");
					sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), showscore ? scYellow : scBlack);
				}
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_DEAD_IDX:
				if(bRecordSee)
					strcpy(acLine,"  (ĭ)");
				else
					strcpy(acLine,"缮 ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), showdead ? scYellow : scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_MOYO_IDX:
				if(bRecordSee)
					strcpy(acLine,"(ĭ)");
				else
					strcpy(acLine," ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_RESIGN_IDX:
				if(bRecordSee)
					strcpy(acLine,"(ĭ)");
				else
					strcpy(acLine," ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_LOAD_IDX:
				if(bRecordSee)
					strcpy(acLine,"(Ǿ)");
				else
					strcpy(acLine,"ҷ ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_SAVE_IDX:
				if(bRecordSee)
					strcpy(acLine,"(ǵ)");
				else
					strcpy(acLine," ϱ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
			case BTN_OPTION_IDX:
				strcpy(acLine,"ȯ ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
  			break;
  		case BTN_QUIT_IDX:
				strcpy(acLine," ϱ");
				sfText = TTF_RenderUTF8_Blended(defaultFont, euckr_to_utf(acLine), scBlack);
  			SDL_BlitSurface(sfText, NULL,sfScreen, &srBtn);
				break;
		}
	}
	SDL_Flip(sfScreen);
}

static Uint32 TimerCallback(Uint32 interval, void *param)
{
#ifdef V43_DEBUG
	printf("timer iMoveNum[%d]\n",iMoveNum);
#endif

	SDL_Event event;
  SDL_UserEvent userevent;

	userevent.type = SDL_USEREVENT;
  userevent.code = 90;//GNU go
  userevent.data1 = NULL;
  userevent.data2 = NULL;

  event.type = SDL_USEREVENT;
  event.user = userevent;

  SDL_PushEvent(&event);
	return interval;
}
/*
 * Check if we have a valid command.
 */

static int
get_command(char *command)
{
  char ch;
  int pos_x;
  int pos_y;
  Uint8 bDone=0;
  int i,j,iSelMenu=0;
    
  /* Check to see if a move was input. */
#ifdef V43_DEBUG
 printf("command in[%s]\n",command);
#endif
  /* Kill leading spaces. */
  while (command[0] == ' ')
    command++;
    
  DrawButton();
  
  while(!bDone)
  {
	  while ( SDL_PollEvent(&event) ) 
		{
			if((event.type==SDL_USEREVENT)&&(event.user.code==90))
			{
				if(iMoveNum<iMaxMoveNum) 
					iMoveNum++;
				else
				{
					iMoveNum=iMaxMoveNum;
					bTimer=false;
			 		SDL_RemoveTimer(timer1);
				}	
				
				sprintf(command,"goto %d",iMoveNum); 
				return CMD_GOTO;
			}
			
			if(event.type==SDL_MOUSEBUTTONDOWN)
			 {
			 		SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_IGNORE);
			 		SDL_EventState(SDL_MOUSEMOTION,SDL_IGNORE);
			 		SDL_EventState(SDL_KEYDOWN,SDL_IGNORE);
			 		//board click
			 		ch='A';
			 		for(i=0;i<board_size;i++)
			 		{
			 			if(bRecordSee) break;//⺸  Ŭȵǰ
			 			
			 			for(j=0;j<board_size;j++)
			 			{
				 			pos_x=POS_BOARD_X+19+i*POS_BOARD_SPACE;
				 			pos_y=POS_BOARD_Y+14+j*POS_BOARD_SPACE;
				 			
				 			if((pos_x < event.button.x) && (pos_x+POS_BOARD_SPACE > event.button.x) \
		 						&& (pos_y < event.button.y) && (pos_y+POS_BOARD_SPACE > event.button.y))
		 						{
		 							if((i+'A')>='I')
		 								ch=ch+i+1;
		 							else
		 								ch=ch+i;
		 							sprintf(command,"%c%d",ch,(board_size-j));

#ifdef V43_DEBUG
	printf("move command [%s]\n",command);
#endif
									SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
									SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
									SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
									return MOVE;
		 						}
			 			}
			 			
			 		}
			 		//button click
			 		for(i=0;i<NUM_BUTTON_CNT;i++)
			 		{
			 			if((srButton[i].x < event.button.x) && (srButton[i].x+srButton[i].w > event.button.x) \
		 				&& (srButton[i].y < event.button.y) && (srButton[i].y+srButton[i].h > event.button.y))
		 				{
		 					//sound click
		 					if(((i==BTN_LOAD_IDX)&&(bRecordSee==false))||((i==BTN_REC_LOAD_IDX)&&(bRecordSee==true)))
		 					{
		 						if(bTimer)
		 						{
			 						bTimer=false;
			 						SDL_RemoveTimer(timer1);
			 					}
		 						return CheckLoadMenu(command);
		 					}
		 					else if((i==BTN_SAVE_IDX)&&(bRecordSee==false))
		 					{
		 						SaveSGFFile(command);
		 						return CMD_SAVE;
		 					}
		 					else
		 					{
		 						if(bRecordSee)
		 						{
		 							switch(i)
		 							{
		 								case BTN_REC_AUTO_IDX	  : 
		 									iBtnCmd[i]=CMD_GOTO;
		 									if(iMoveNum<iMaxMoveNum) 
												iMoveNum++;
											sprintf(command,"goto %d",iMoveNum);
		 									break;
										//case BTN_REC_LOAD_IDX		: iBtnCmd[i]=; break;
										case BTN_REC_LONE_IDX	  : 
											iBtnCmd[i]=CMD_GOTO; 
											if(iMoveNum>1) iMoveNum--;
											sprintf(command,"goto %d",iMoveNum); 
											break;
										case BTN_REC_RONE_IDX	  : 
											iBtnCmd[i]=CMD_GOTO;
											if(iMoveNum<iMaxMoveNum) iMoveNum++;
											sprintf(command,"goto %d",iMoveNum); 
											break;
										case BTN_REC_LTEN_IDX   : 
											iBtnCmd[i]=CMD_GOTO; 
											iMoveNum=iMoveNum-10;
											if(iMoveNum<0) iMoveNum=1;
											sprintf(command,"goto %d",iMoveNum);
											break;
										case BTN_REC_RTEN_IDX	  : 
											iBtnCmd[i]=CMD_GOTO; 
											iMoveNum=iMoveNum+10;
											if(iMoveNum>iMaxMoveNum) iMoveNum=iMaxMoveNum;
											sprintf(command,"goto %d",iMoveNum);
											break;
										case BTN_REC_FIRST_IDX	: 
											iBtnCmd[i]=CMD_GOTO; 
											iMoveNum=1;
											sprintf(command,"goto %d",iMoveNum);
											break;
										case BTN_REC_LAST_IDX	  : 
											iBtnCmd[i]=CMD_LAST; 
											iMoveNum=iMaxMoveNum;
											break;
										case BTN_REC_OPTION_IDX : iBtnCmd[i]=RESIGN; bSetOption=true; break;
										case BTN_REC_QUIT_IDX		: iBtnCmd[i]=QUIT; break;

		 							}
		 							
		 							if((i==BTN_REC_AUTO_IDX)&&(bTimer==false))
		 							{
		 								bTimer=true;
				 						SDL_RemoveTimer(timer1);
				 						//SDL_SetTimer(0,NULL);
				 						//SDL_SetTimer(iSecond*1000, &TimerCallback);
				 						timer1=SDL_AddTimer(iSecond*1000, &TimerCallback,0);
		 							}
		 							else
		 							{
		 								bTimer=false;
		 								SDL_RemoveTimer(timer1);
		 								//SDL_SetTimer(0,NULL);
		 							}
		 						}
		 						else
		 						{
			 						switch(i)
			 						{
			 							case BTN_BACK_IDX 	: iBtnCmd[i]=CMD_BACK; break;
			 							case BTN_PASS_IDX		: iBtnCmd[i]=PASS; break;
			 							case BTN_SCORE_IDX	: iBtnCmd[i]=SCORE; break;
			 							case BTN_DEAD_IDX		: iBtnCmd[i]=CMD_DEAD; break;
			 							case BTN_MOYO_IDX		: iBtnCmd[i]=CMD_SHOWMOYO; break;
			 							case BTN_RESIGN_IDX	: iBtnCmd[i]=RESIGN; break;
			 							case BTN_OPTION_IDX		:	
			 								iBtnCmd[i]=RESIGN; 
			 								bSetOption=true;
			 								break;  
			 							case BTN_QUIT_IDX		:	iBtnCmd[i]=QUIT; break;
			 							//case BTN_LOAD_IDX		: iBtnCmd[i]=; break;
			 							//case BTN_SAVE_IDX		: iBtnCmd[i]=; break;
			 						}
			 					}
		 						iSelMenu = iBtnCmd[i];
#ifdef V43_DEBUG
	printf("button [%d]\n",iSelMenu);
#endif
		 				 		SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
		 				 		SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
		 				 		SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
		 						return iSelMenu;
		 					}
		 				}
			 		}
			 }
		}
		 SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
		 SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);
		 SDL_EventState(SDL_KEYDOWN,SDL_ENABLE);
		 SDL_Delay(100);  
	}
	
	//return iSelMenu;
  /* Check to see if a move was input. */
  /*
  if (!((sscanf(command, "%c%d", &c, &d) != 2)
	|| ((c = toupper((int) c)) < 'A')
	|| ((c = toupper((int) c)) > 'Z')
	|| (c == 'I')))
    return MOVE;
	*/
  /* help shortcut */
  //if (command[0] == '?')
  //  return HELP;

  /* Kill leading spaces. */
  //while (command[0] == ' ')
  //  command++;
/*
  if (!strncmp(command, "playblack", 9)) return PLAYBLACK;
  if (!strncmp(command, "playwhite", 9)) return PLAYWHITE;
  if (!strncmp(command, "showboard", 9)) return SHOWBOARD;
  if (!strncmp(command, "showdragons", 9)) return CMD_SHOWDRAGONS;
  if (!strncmp(command, "listdragons", 9)) return CMD_LISTDRAGONS;
  if (!strncmp(command, "boardsize", 9)) return SETBOARDSIZE;
  if (!strncmp(command, "freehandicap", 9)) return FREEHANDICAP;
  if (!strncmp(command, "handicap", 5)) return SETHANDICAP;
  if (!strncmp(command, "display", 7)) return DISPLAY;
  if (!strncmp(command, "helpdebug", 7)) return CMD_HELPDEBUG;
  if (!strncmp(command, "resign", 6)) return RESIGN;
  if (!strncmp(command, "showmoyo", 6)) return CMD_SHOWMOYO;
  if (!strncmp(command, "showterri", 6)) return CMD_SHOWTERRI;
  if (!strncmp(command, "showarea", 6)) return CMD_SHOWAREA;
  if (!strncmp(command, "depth", 5)) return SETDEPTH;
  if (!strncmp(command, "switch", 5)) return SWITCH;
  if (!strncmp(command, "komi", 4)) return SETKOMI;
  if (!strncmp(command, "play", 4)) return PLAY;
  if (!strncmp(command, "info", 4)) return INFO;
  if (!strncmp(command, "force", 4)) return FORCE;
  if (!strncmp(command, "hurry", 5)) return SETHURRY;
  if (!strncmp(command, "level", 5)) return SETLEVEL;
  if (!strncmp(command, "pass", 4)) return PASS;
  if (!strncmp(command, "save", 3)) return CMD_SAVE;
  if (!strncmp(command, "load", 3)) return CMD_LOAD;
  if (!strncmp(command, "end", 3)) return END;
  if (!strncmp(command, "move", 3)) return MOVE;
  if (!strncmp(command, "undo", 3)) return UNDO;
  if (!strncmp(command, "comment", 3)) return COMMENT;
  if (!strncmp(command, "score", 3)) return SCORE;
  if (!strncmp(command, "dead", 3)) return CMD_DEAD;
  if (!strncmp(command, "capture", 3)) return CMD_CAPTURE;
  if (!strncmp(command, "defend", 3)) return CMD_DEFEND;
  if (!strncmp(command, "exit", 4)) return EXIT;
  if (!strncmp(command, "quit", 4)) return QUIT;
  if (!strncmp(command, "help", 1)) return HELP;
  if (!strncmp(command, "back", 2)) return CMD_BACK;
  if (!strncmp(command, "forward", 2)) return CMD_FORWARD;
  if (!strncmp(command, "last", 2)) return CMD_LAST;
  if (!strncmp(command, "goto", 2)) return CMD_GOTO;
  if (!strncmp(command, "game", 2)) return NEW;
  if (!strncmp(command, "count", 3)) return COUNT;
  if (!strncmp(command, "continue", 4)) return CONTINUE;
*/
  /* No valid command was found. */
  return INVALID;
}


/*
 * Make a move.
 */

static int
do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
{
  int i, j;

  if (!string_to_location(board_size, command, &i, &j)) {
    printf("\nInvalid move: %s\n", command);
    return 0;
  }
  if (!gnugo_is_legal(i, j, gameinfo->to_move)) {
    printf("\nIllegal move: %s", command);
    return 0;
  }
  //*passes = 0;
  
  TRACE("\nyour move: %m\n\n", i, j);
  init_sgf(gameinfo);
  gnugo_play_move(i, j, gameinfo->to_move);
  sgffile_add_debuginfo(sgftree.lastnode, 0);
  sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
  sgffile_output(&sgftree);
  last_move_i = i;
  last_move_j = j;
  
  if(bSound)
	{
		PlayWav(mcStone,0); 
		SDL_Delay(500);
	}
  //if (opt_showboard && !emacs) {
  if (opt_showboard) {
    ascii_showboard(gameinfo);
#ifdef V43_DEBUG
    printf("Com is thinking...\n");
#endif
    DrawMessage(" ...",true,0);
  }
  
  if (force) {
    gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
    gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
    sgftreeAddComment(&sgftree, "forced");
    return 0;
  }
  
  gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
  return computer_move(gameinfo, passes);
}


/*
 * Make a pass.
 */

static int
do_pass(Gameinfo *gameinfo, int *passes, int force)
{
  (*passes)++;
  init_sgf(gameinfo);
  gnugo_play_move(-1, -1, gameinfo->to_move);
  sgffile_add_debuginfo(sgftree.lastnode, 0);
  sgftreeAddPlay(&sgftree, gameinfo->to_move, -1, -1);
  sgffile_output(&sgftree);

  gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
  if (force) {
    gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
    sgftreeAddComment(&sgftree, "forced");
    return 0;
  }

  return computer_move(gameinfo, passes);
}


void LoadConfig()
{
		int i,j;
		FILE *fp;
	  //read configure file 
	  MakeFullFilePath(szIniPath,"baduk_option.ini",acFullPath);

		if((fp = fopen(acFullPath,"r")) != NULL)
		{
			char *pszToken;
			char szSeps[]="=";
			char acLine[16];
			int ch = ';';//Ṯ 59
			char *pLineFeed;
			char acOption[16];
			//LEVEL=10;
			//HANDICAP=0;
			//ADVANTAGE=6.5;
			//BOARDSIZE=19;
			//SOUND=1;
			//TIMER=5;
			
			for(i=0;i<NUM_OPTION_CNT-1;i++)
			{
				if(fgets(acLine, 16, fp) == NULL)
						break;
				pszToken = strtok( acLine, szSeps );
				if(pszToken==NULL) 
					break;
				
				for( j=0;j<2;j++ )
				{
					if(j==1)
					{
						memset(acOption,0x00,16);
						pLineFeed = strrchr( pszToken, ch );
						if(pLineFeed!=NULL)
						{
							memcpy(acOption,pszToken,pLineFeed-pszToken);
							acOption[pLineFeed-pszToken] ='\0';
							
							switch(i)
							{
								case OPT_LEVEL_IDX:
									iLevel=atoi(acOption);
									printf("\nSet level to %d\n", iLevel);
									if(iLevel>10) iLevel=10;
									else if(iLevel<0) iLevel=0;
									break;
								case OPT_HANDICAP_IDX:
									iHandiCap = atoi(acOption);
									if (iHandiCap < 0 || iHandiCap > MAX_HANDICAP) {
								    printf("\nInvalid handicap: %d\n", iHandiCap);
								    iHandiCap = 0;
								    //break;
								  }
									break;
								case OPT_ADVANTAGE_IDX:
									fSetKomi=atof(acOption);
									printf("\nSet Komi to %.1f\n", fSetKomi);
									if(fSetKomi==0.0) fSetKomi=6.5;
									if(iHandiCap>0) bSetKomi=false;
									break;
								case OPT_BOARDSIZE_IDX: 
									iBoardSize = atoi(acOption);
									if (iBoardSize < MIN_BOARD || iBoardSize > MAX_BOARD) {
								    printf("\nInvalid board size: %d\n", iBoardSize);
								    iBoardSize=19;
								    //break;
								  }
								  break;
								case OPT_SOUND_IDX:
									bSound=atoi(acOption);
									printf("\nSet Sound to %d\n", bSound);
									break;
								case OPT_TIMER_IDX: 
									iSecond = atoi(acOption);
									if (iSecond < 1 || iSecond > 60) {
								    printf("\nInvalid Second: %d\n", iSecond);
								    iSecond=2;
								  }
								  break;
							}
							
#ifdef V43_DEBUG
		printf("loadconfig[%s]\n",acOption);
#endif
						}
						
					}
					pszToken = strtok( NULL, szSeps );
				}
			}
			
			fclose(fp);
		}
}

void SaveConfig(Gameinfo *gameinfo)
{
	//read configure file
	MakeFullFilePath(szIniPath,"baduk_option.ini",acFullPath);
	FILE *fp;
	int i;
	if((fp = fopen(acFullPath,"w")) != NULL)
	{
		char acLine[16];
		//LEVEL=10;
		//HANDICAP=0;
		//ADVANTAGE=6.5;
		//BOARDSIZE=19;
		//SOUND=1;
		//TIMER=5;
		
		for( i=0;i<NUM_OPTION_CNT-1;i++ )  
		{
			memset(acLine,0x00,16);
			switch(i)
			{
				case OPT_LEVEL_IDX:
					sprintf(acLine,"LEVEL=%d;\n",level);
					fputs(acLine, fp);
					break;
				case OPT_HANDICAP_IDX:
					sprintf(acLine,"HANDICAP=%d;\n",gameinfo->handicap);
					fputs(acLine, fp);
					break;
				case OPT_ADVANTAGE_IDX:
					sprintf(acLine,"ADVANTAGE=%.1f;\n",fSetKomi);
					fputs(acLine, fp);
					break;
				case OPT_BOARDSIZE_IDX:
					sprintf(acLine,"BOARDSIZE=%d;\n",board_size);
					fputs(acLine, fp);
					break;
				case OPT_SOUND_IDX:
					sprintf(acLine,"SOUND=%d;\n",bSound);
					fputs(acLine, fp);
					break;
				case OPT_TIMER_IDX:
					sprintf(acLine,"TIMER=%d;\n",iSecond);
					fputs(acLine, fp);
					break;
			}
			
#ifdef V43_DEBUG
		printf("SaveConfig[%s]\n",acLine);
#endif
		}
		
		fclose(fp);
	}
}

/*
 * Play a game against an ascii client.
 */

void
play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
{
  int m, num;
  int sz;
  float fnum;
  int passes = 0;  /* two passes and its over */
  int tmp;
  char line[80];
  char *line_ptr = line;
  char *command;
  char *tmpstring;
  int state = 1;//0:,1:,2:,-1:?
  
  LoadAllWav();
  LoadConfig();
    
  setvbuf(stdout, (char *)NULL, _IONBF, 0); /* No buffering. */
  
  sgftree = *tree;

  if (filename) {
    gameinfo_load_sgfheader(gameinfo, sgftree.root);
    gameinfo->to_move = gameinfo_play_sgftree(gameinfo, &sgftree, until);
    sgf_initialized = 1;
  }
  else {
    if (sgfGetIntProperty(sgftree.root, "SZ", &sz)) 
      gnugo_clear_board(sz);
    if (gameinfo->handicap == 0)
      gameinfo->to_move = BLACK;
    else {
      gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
      gameinfo->to_move = WHITE;
    }
    sgf_initialized = 0;
  }
	
	
  while (state == 1) {
    state = 0;

    /* No score is estimated yet. */
    current_score_estimate = NO_SCORE;

    /* Allow resignation at interface level (the engine may still be not
     * allowed to resign.
     */
    resignation_allowed = 1;
    
    if(bSetOption)
		{
			//۽ ɼǼ
			CheckOptionMenu(gameinfo,&state,&passes);
			bSetOption=false;
		}
#ifdef V43_DEBUG
    printf("\nBeginning ASCII mode game.\n\n");
    gameinfo_print(gameinfo);
#endif
    /* Does the computer play first?  If so, make a move. */
    if ((gameinfo->computer_player == gameinfo->to_move)&&(bForce==false)&&(bRecordSee==false))
      state = computer_move(gameinfo, &passes);

    /* main ASCII Play loop */
    while (state == 0) {
      /* Display game board. */
      if (opt_showboard)
				ascii_showboard(gameinfo);
      
      /* Print the prompt */
      //mprintf("%s(%d): ", color_to_string(gameinfo->to_move), movenum + 1);
			DrawInfo(INFO_TOMOVE,gameinfo);
      /* Read a line of input. */
      line_ptr = line;
      //command = strtok(line_ptr, "");
      /*
      if (!fgets(line, 80, stdin)) {
	printf("\nThanks! for playing Baduk.\n\n");
	return;
      }
			*/
      while (state == 0 && (command = strtok(line_ptr, ";"), line_ptr = 0, command)) {
      //while (state == 0) {
 printf("get_command\n");
	/* Get the command or move. */
	switch (get_command(command)) {
	case RESIGN:
	  state = ascii_endgame(gameinfo, 1);
	  break;

	case END:
	case EXIT:
	case QUIT:
	  //printf("\nThanks! for playing Baduk.\n\n");
	  DrawMessage("ſ̳?\nε ż մϴ.",true,1000);
	  SaveConfig(gameinfo);
	  FreeAllWav();
	  return;
/*
	case HELP:
	  show_commands();
	  break;

	case CMD_HELPDEBUG:
	  printf(DEBUG_COMMANDS);
	  break;

	case SHOWBOARD:
	  opt_showboard = !opt_showboard;
	  break;

	case INFO:
	  printf("\n");
	  gameinfo_print(gameinfo);
	  break;

	case SETBOARDSIZE:
	  if (sgf_initialized) {
	    printf("Boardsize cannot be changed after record is started!\n");
	    break;
	  }
	  command += 10;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  if (num < MIN_BOARD || num > MAX_BOARD) {
	    printf("\nInvalid board size: %d\n", num);
	    break;
	  }
	  sz = num;
	  /- Init board. -/
	  gnugo_clear_board(sz);
	  /- In case max handicap changes on smaller board. -/
	  gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
	  sgfOverwritePropertyInt(sgftree.root, "SZ", sz);
	  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
	  break;

	case SETHANDICAP:
	  if (sgf_initialized) {
	    printf("Handicap cannot be changed after game is started!\n");
	    break;
	  }
	  command += 9;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  if (num < 0 || num > MAX_HANDICAP) {
	    printf("\nInvalid handicap: %d\n", num);
	    break;
	  }
	  /- Init board. -/
	  gnugo_clear_board(board_size);
	  /- Place stones on board but don't record sgf 
	   * in case we change more info. -/
	  gameinfo->handicap = gnugo_placehand(num);
	  printf("\nSet handicap to %d\n", gameinfo->handicap);
          gameinfo->to_move = (gameinfo->handicap ? WHITE : BLACK);
	  break;

	case FREEHANDICAP:
	  if (sgf_initialized) {
	    printf("Handicap cannot be changed after game is started!\n");
	    break;
	  }
	  while (*command && *command != ' ')
	    command++;
	  ascii_free_handicap(gameinfo, command);
	  break;

	case SETKOMI:
	  if (sgf_initialized) {
	    printf("Komi cannot be modified after game record is started!\n");
	    break;
	  }
	  command += 5;
	  if (sscanf(command, "%f", &fnum) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  komi = fnum;
	  printf("\nSet Komi to %.1f\n", komi);
	  break;

	case SETDEPTH:
	  command += 6;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  mandated_depth = num;
	  printf("\nSet depth to %d\n", mandated_depth);
	  break;

	case SETLEVEL:
	  command += 6;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  level = num;
	  printf("\nSet level to %d\n", level);
	  break;
*/
	  /* Level replaces hurry as of 2.7.204. This option is retained
	   * for compatibility with gnugoclient. 
	   */
/*
	case SETHURRY:
	  command += 6;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  level = 10 - num;
	  printf("\nSet hurry to %d\n", 10 - level);
	  break;
*/
	case DISPLAY:
	  if (!opt_showboard)
	    ascii_showboard(gameinfo);
	  break;
/*
	case FORCE:
	
	
	  command += 6; /- skip the force part... -/
	  switch (get_command(command)) {
	  case MOVE:
	    state = do_move(gameinfo, command, &passes, 1);
	    break;
	  case PASS:
	    state = do_pass(gameinfo, &passes, 1);
	    break;
	  default:
	    printf("Illegal forced move: %s %d\n", command,
		   get_command(command));
	    break;
	  }
	  */
	  break;

	case MOVE:
	  state = do_move(gameinfo, command, &passes, bForce);
#ifdef V43
	  ascii_showboard(gameinfo);//@v43 ʿ
#endif
	  break;

	case PASS:
		if(passes<2)
	  	DrawMessage("ι нϸ 뱹 \n˴ϴ.",true,1000);
	  state = do_pass(gameinfo, &passes, bForce);
	  break;
/*
	case PLAY:
	  command += 5;
	  if (sscanf(command, "%d", &num) != 1) {
	    printf("\nInvalid command syntax!\n");
	    break;
	  }
	  if (num >= 0)
	    for (m = 0; m < num; m++) {
	      gameinfo->computer_player 
		= OTHER_COLOR(gameinfo->computer_player);
	      state = computer_move(gameinfo, &passes);
	      if (state)
		break;
	      if (passes >= 2)
		break;
	    }
	  else {
	    printf("\nInvalid number of moves specified: %d\n", num);
	    break;
	  }
	  break;

	case PLAYBLACK:
	  if (gameinfo->computer_player == WHITE)
	    gameinfo->computer_player = BLACK;
	  if (gameinfo->computer_player == gameinfo->to_move)
	    state = computer_move(gameinfo, &passes);
	  break;

	case PLAYWHITE:
	  if (gameinfo->computer_player == BLACK)
	    gameinfo->computer_player = WHITE;
	  if (gameinfo->computer_player == gameinfo->to_move)
	    state = computer_move(gameinfo, &passes);
	  break;

	case SWITCH:
	  gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
	  state = computer_move(gameinfo, &passes);
	  break;
*/
	case UNDO:
	case CMD_BACK:
	  if (gnugo_undo_move(1)) {
            sgftreeAddComment(&sgftree, "undone");
	    sgftreeBack(&sgftree);
	    gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
	    DrawMessage(" ",true,0);
	  }
	  else
	  {
	  	DrawMessage("   ϴ.",true,1000);
#ifdef V43_DEBUG	  	
	    printf("\nCan't undo.\n");
#endif
	  }
	  break;

	case CMD_FORWARD:
         if (sgftreeForward(&sgftree))
           gameinfo->to_move = gnugo_play_sgfnode(sgftree.lastnode,
						  gameinfo->to_move);
	  else
		{
	  	DrawMessage(" Ʈ Դϴ.",true,1000);
#ifdef V43_DEBUG	
	    printf("\nEnd of game tree.\n");
#endif
	  }
	  break;

	case CMD_LAST:
         while (sgftreeForward(&sgftree))
           gameinfo->to_move = gnugo_play_sgfnode(sgftree.lastnode,
						  gameinfo->to_move);
	  break;
/*
	case COMMENT:
	  printf("\nEnter comment. Press ENTER when ready.\n");
	  fgets(line, 80, stdin);
	  sgftreeAddComment(&sgftree, line);
	  break;
*/
	case SCORE:
	  showscore = !showscore;
	  if (!showscore)
	    current_score_estimate = NO_SCORE;
	  break;

	case CMD_DEAD:
		if(!showdead)
			DrawMessage("缮 ˻մϴ.",true,0);
	  examine_position(FULL_EXAMINE_DRAGONS);
	  showdead = !showdead;
	  if(showdead)
	  {
	  	DrawMessage(" ",false,0);
		}
		ascii_showboard(gameinfo);
	  break;

	case CMD_CAPTURE:
	  strtok(command, " ");
	  showcapture(strtok(NULL, " "));
	  break;

	case CMD_DEFEND:
	  strtok(command, " ");
	  showdefense(strtok(NULL, " "));
	  break;

	case CMD_SHOWMOYO:
	  tmp = printmoyo;
	  printmoyo = PRINTMOYO_MOYO;
	  examine_position(EXAMINE_DRAGONS);
	  print_moyo();
	  printmoyo = tmp;
	  
	  ascii_showboard(gameinfo);
	  break;

	case CMD_SHOWTERRI:
	  tmp = printmoyo;
	  printmoyo = PRINTMOYO_TERRITORY;
	  examine_position(EXAMINE_DRAGONS);
	  print_moyo();
	  printmoyo = tmp;
	  break;

	case CMD_SHOWAREA:
	  tmp = printmoyo;
	  printmoyo = PRINTMOYO_AREA;
	  examine_position(EXAMINE_DRAGONS);
	  print_moyo();
	  printmoyo = tmp;
	  break;

	case CMD_SHOWDRAGONS:
	  examine_position(EXAMINE_DRAGONS);
	  showboard(1);
	  break;

	case CMD_GOTO:
	  strtok(command, " ");
	  ascii_goto(gameinfo, strtok(NULL, " "));
	  break;

	case CMD_SAVE:
	  strtok(command, " ");
	  tmpstring = strtok(NULL, " ");
	  if (tmpstring) {
	    /* discard newline */
	    tmpstring[strlen(tmpstring)] = 0;
	    /* make sure we are saving proper handicap */
	    init_sgf(gameinfo);
	    writesgf(sgftree.root, tmpstring);
	    
	    printf("You may resume the game");
	    //printf(" with -l %s --mode ascii\n", tmpstring);
	    //printf("or load %s\n", tmpstring);
	  }
	  else
	    printf("Please specify filename\n");
	  break;

	case CMD_LOAD:
	  strtok(command, " ");
	  tmpstring = strtok(NULL, " ");
	  if (tmpstring) {
	    /* discard newline */
	    tmpstring[strlen(tmpstring)] = 0;
	    if (!sgftree_readfile(&sgftree, tmpstring)) {
	      fprintf(stderr, "Cannot open or parse '%s'\n", tmpstring);
	      break;
	    }
            /* to avoid changing handicap etc. */
            sgf_initialized = 1;
            gameinfo_load_sgfheader(gameinfo, sgftree.root);
            gameinfo->to_move = gameinfo_play_sgftree(gameinfo, &sgftree, NULL);
            if(bRecordSee==false)
            	gameinfo->computer_player = OTHER_COLOR(gameinfo->to_move);
#ifdef V43_DEBUG
	printf("gameinfo->to_move[%d]\n",gameinfo->to_move);
#endif
	    sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
	  }
	  else
	    printf("Please specify a filename\n");
	    
	  if(bRecordSee)
	  {
	  	iMoveNum=movenum+1;
	  	iMaxMoveNum=iMoveNum;
#ifdef V43_DEBUG
	printf("iMaxMoveNum[%d]\n",iMaxMoveNum);
#endif
	  }
	  break;

	case CMD_LISTDRAGONS:
	  examine_position(EXAMINE_DRAGONS);
	  show_dragons();
	  break;

	default:
		//DrawMessage("߸ Դϴ.",true);
	  printf("\nInvalid command: %s", command);
	  break;
	}
#ifdef V43_DEBUG
	printf("passes[%d]\n",passes);
#endif
	if (passes >= 2)
	  state = ascii_endgame(gameinfo, 0);
      }
    }

    sgffile_output(&sgftree);
    passes = 0;
    //reset
    showdead = 0;
    DrawMessage("뱹 Ͽϴ.",true,0);
    /* Play a different game next time. */
    update_random_seed();

    /* Free the sgf tree and prepare for a new game. */
    sgfFreeNode(sgftree.root);
    sgftree_clear(&sgftree);
    sgftreeCreateHeaderNode(&sgftree, board_size, komi);
    sgf_initialized = 0;

    gameinfo_clear(gameinfo, board_size, komi);
  }

  //printf("\nThanks! for playing Baduk.\n\n");
  DrawMessage("ſ̳?\nε ż մϴ.",true,1000);
  SaveConfig(gameinfo);
  FreeAllWav();
}

void
play_ascii_emacs(SGFTree *tree, Gameinfo *gameinfo,
		 char *filename, char *until)
{
  emacs = 1;
  play_ascii(tree, gameinfo, filename, until);
}


/* Communicates with user after a game has ended. */
static int
ascii_endgame(Gameinfo *gameinfo, int reason)
{
  char line[80];
  char *line_ptr;
  char *command;
  char *tmpstring;
  int state = 0;

  if (reason == 0) {		/* Two passes, game is over. */
  	DrawMessage("뱹 Ǿϴ.\n谡մϴ.",true,1000);  
  	gnugo_who_wins(gameinfo->computer_player, stdout);
    
    printf("\nIf you disagree, we may count the game together.\n");

    sgftreeWriteResult(&sgftree, estimate_score(NULL, NULL), 1);
  }
  else {
    int color = OTHER_COLOR(gameinfo->to_move);

    if (reason == 1)		/* Our opponent has resigned. */
    {	
    	if((bForce==true)||(bRecordSee==true))
    		DrawMessage(" Ұ",true,1000);
    	else
      	DrawMessage(" Ұ",true,1000);
    }
    else			/* We have resigned. */
      DrawMessage(" Ұ",true,1000);

    printf("Result: %c+Resign\n\n", color == WHITE ? 'W' : 'B');
    sgftreeWriteResult(&sgftree, color == WHITE ? 1000.0 : -1000.0, 1);
  }
/*
  while (state == 0) {
    printf("You may optionally save the game as an SGF file.\n\n");
    printf("Type \"save <filename>\" to save,\n");
    if (reason == 0)
      printf("     \"count\" to recount,\n");
    else if (reason == 2)
      printf("     \"continue\" to decline resignation and continue the game,\n");
    printf("     \"quit\" to quit\n");
    printf(" or  \"game\" to play again\n");

    line_ptr = line;
    if (!fgets(line, 80, stdin))
      break;

    command = strtok(line_ptr, "");
    switch (get_command(command)) {
    case CMD_SAVE:
      strtok(command, " ");
      tmpstring = strtok(NULL, " ");
      if (tmpstring) {
	/- discard newline -/
	tmpstring[strlen(tmpstring)-1] = 0;
	init_sgf(gameinfo);
	writesgf(sgftree.root, tmpstring);
      }
      else
	printf("Please specify filename\n");
      break;

    case COUNT:
      if (reason == 0)
	ascii_count(gameinfo);
      break;

    case CONTINUE:
      state = -1;
      break;

    case NEW:
      state = 1;
      break;

    case QUIT:
      state = 2;
      break;

    default:
      state = 0;
    }
  }
*/
	//if (reason == 0)
	//	ascii_count(gameinfo);
	state = 1;
  return state;
}


/* ascii_count() scores the game.
 */
static void
ascii_count(Gameinfo *gameinfo)
{
  char line[12];
  int done = 0;
  int i, j;
  int xyzzy = 1;
  
  DrawMessage("뱹 Ǿϴ.\nմϴ.",true,1000);  
  showdead = 1;
	ascii_showboard(gameinfo);
  /*
  while (!done) {
    printf("Dead stones are marked with small letters (x,o).\n");
    printf("\nIf you don't agree, enter the location of a group\n");
    printf("to toggle its state or \"done\".\n");

    if (!fgets(line, 12, stdin))
      return; /- EOF or some error -/
    
    for (i = 0; i < 12; i++)
      line[i] = tolower((int) line[i]);
    if (!strncmp(line, "done", 4))
      done = 1;
    else if (!strncmp(line, "quit", 4))
      return;
    else if (!strncmp(line, "xyzzy", 5)) {
      if (xyzzy) {
	printf("\nYou are in a debris room filled with stuff washed in from the\n");
	printf("surface.  A low wide passage with cobbles becomes plugged\n");
	printf("with mud and debris here, but an awkward canyon leads\n");
	printf("upward and west.  A note on the wall says:\n");
	printf("   Magic Word \"XYZZY\"\n\n");
	xyzzy = 0;
      }
      else {
	printf("You are inside a building, a well house for a large spring.\n\n");
	xyzzy = 1;
      }
    }
    else if (!strncmp(line, "help", 4)) {
      printf("\nIf there are dead stones on the board I will remove them.\n");
      printf("You must tell me where they are. Type the coordinates of a\n");
      printf("dead group, or type \"done\"\n");
    }      
    else if (!strncmp(line, "undo", 4)) {
      printf("UNDO not allowed anymore. The status of the stones now\n");
      printf("toggles after entering the location of it.\n");
      ascii_showboard();
    }
    else {
      if (!string_to_location(board_size, line, &i, &j)
	  || BOARD(i, j) == EMPTY)
	printf("\ninvalid!\n");
      else {
	int status = dragon_status(POS(i, j));
	status = (status == DEAD) ? ALIVE : DEAD;
	change_dragon_status(POS(i, j), status);
	ascii_showboard();
      }
    }
  }
  */
  gnugo_who_wins(gameinfo->computer_player, stdout);
}


static void
showcapture(char *line)
{
  int i, j, x, y;
  if (line)
    if (!string_to_location(board_size, line, &i, &j)
	|| BOARD(i, j) == EMPTY) {
      printf("\ninvalid point!\n");
      return;
    }
  
  if (gnugo_attack(i, j, &x, &y))
    mprintf("\nSuccessfull attack of %m at %m\n", i, j, x, y);
  else
    mprintf("\n%m cannot be attacked\n", i, j);
}


static void
showdefense(char *line)
{
  int i, j, x, y;
  if (line)
    if (!string_to_location(board_size, line, &i, &j)
	|| BOARD(i, j) == EMPTY) {
      printf("\ninvalid point!\n");
      return;
    }

    if (gnugo_attack(i, j, &x, &y)) {
      if (gnugo_find_defense(i, j, &x, &y))
        mprintf("\nSuccessfull defense of %m at %m\n", i, j, x, y);
      else
        mprintf("\n%m cannot be defended\n", i, j);
    }
    else
      mprintf("\nThere is no need to defend %m\n", i, j);
}


static void
ascii_goto(Gameinfo *gameinfo, char *line)
{
  const char *movenumber = line;

  if (!line)
    return;
  
  if (!strncmp(line, "last", 4))
    movenumber = "9999";
  else if (!strncmp(line, "first", 4))
    movenumber = "1";
  
  printf("goto %s\n", movenumber);
  gameinfo_play_sgftree(gameinfo, &sgftree, movenumber);
}


static void
ascii_free_handicap(Gameinfo *gameinfo, char *handicap)
{
  int handi;
  int i;
  char line[80];
  int stones[MAX_BOARD*MAX_BOARD];
  int x, y, pos;

  if (sscanf(handicap, "%d", &handi) == 1) {
    /* GNU Go is to place handicap */
    if (handi < 0 || handi == 1) {
      printf("\nInvalid command syntax!\n");
      return;
    }

    gnugo_clear_board(board_size);
    handi = place_free_handicap(handi);
    printf("\nPlaced %d stones of free handicap.\n", handi);
  }
  else { /* User is to place handicap */
    gnugo_clear_board(board_size);
    handi = 0;

    while (1) {
      ascii_showboard(gameinfo);
      printf("\nType in coordinates of next handicap stone, or one of the following commands:\n");
      printf("  undo        take back the last stone placed\n");
      printf("  clear       remove all the stones placed so far\n");
      printf("  done        finish placing handicap\n\n");
      printf("You have placed %d handicap stone(s) so far.\n\n", handi);

      if (!fgets(line, 80, stdin))
        return; /* EOF or some error */
      for (i = 0; i < 80; i++)
        line[i] = tolower((int) line[i]);

      if (!strncmp(line, "undo", 4)) {
        if (!handi)
	  printf("\nNothing to undo.\n");
	else {
	  remove_stone(stones[--handi]);
	  gprintf("\nRemoved the stone at %m.\n", I(stones[handi]),
		  J(stones[handi]));
	}
      }
      else if (!strncmp(line, "clear", 5)) {
        gnugo_clear_board(board_size);
        handi = 0;
      }
      else if (!strncmp(line, "done", 4)) {
	if (handi == 1) /* Don't bother with checks later */
	  printf("\nHandicap cannot be one stone. Either add "
		 "some more, or delete the only stone.\n");
	else
	  break;
      }
      else if (string_to_location(board_size, line, &x, &y)) {
	pos = POS(x, y);
	if (board[pos] != EMPTY)
	  printf("\nThere's already a stone there.\n");
	else {
	  add_stone(pos, BLACK);
	  stones[handi++] = pos;
	}
      }
      else
	printf("\nInvalid command: %s", line);
    }
  }
  gameinfo->handicap = handi;
  gameinfo->to_move = (handi ? WHITE : BLACK);
}


/*
 * Local Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
